@@ -426,15 +426,15 @@ rewind_transport                       (GtkWidget       *widget,
                                         gpointer         user_data);
 
 void
-on_autobutton_l_toggled                 (GtkToggleButton *togglebutton,
+on_autobutton_1_toggled                 (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 void
-on_autobutton_m_toggled                 (GtkToggleButton *togglebutton,
+on_autobutton_2_toggled                 (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 void
-on_autobutton_h_toggled                 (GtkToggleButton *togglebutton,
+on_autobutton_3_toggled                 (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 gboolean
@@ -491,6 +491,76 @@ on_scene6_eventbox_button_press_event  (GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data);
 
+gboolean
+on_scene7_eventbox_button_press_event  (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene8_eventbox_button_press_event  (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene9_eventbox_button_press_event  (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene10_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene11_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene12_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene13_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene14_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene15_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene16_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene17_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene18_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene19_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene20_eventbox_button_press_event (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
 void
 on_setscene_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -499,30 +569,6 @@ void
 on_clearscene_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_scene1_name_changed                 (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_scene2_name_changed                 (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_scene3_name_changed                 (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_scene4_name_changed                 (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_scene5_name_changed                 (GtkEditable     *editable,
-                                        gpointer         user_data);
-
-void
-on_scene6_name_changed                 (GtkEditable     *editable,
-                                        gpointer         user_data);
-
 void
 on_help_button_clicked                 (GtkButton       *button,
                                         gpointer         user_data);
@@ -598,12 +644,6 @@ on_spectrum_options_eventbox_enter_notify_event
                                         GdkEventCrossing *event,
                                         gpointer         user_data);
 
-gboolean
-on_status_label_eventbox_enter_notify_event
-                                        (GtkWidget       *widget,
-                                        GdkEventCrossing *event,
-                                        gpointer         user_data);
-
 gboolean
 on_transport_controls_eventbox_enter_notify_event
                                         (GtkWidget       *widget,
@@ -631,16 +671,6 @@ on_window1_key_press_event             (GtkWidget       *widget,
                                         GdkEventKey     *event,
                                         gpointer         user_data);
 
-gboolean
-on_scene_name_focus_in_event          (GtkWidget       *widget,
-                                       GdkEventFocus   *event,
-                                       gpointer         user_data);
-
-gboolean
-on_scene_name_focus_out_event         (GtkWidget       *widget,
-                                       GdkEventFocus   *event,
-                                       gpointer         user_data);
-
 void
 on_new1_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -829,18 +859,6 @@ pause_transport_toggle                 (GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data);
 
-void
-on_low_balance_scale_value_changed         (GtkRange        *range,
-                                        gpointer         user_data);
-
-void
-on_mid_balance_scale_value_changed         (GtkRange        *range,
-                                        gpointer         user_data);
-
-void
-on_high_balance_scale_value_changed        (GtkRange        *range,
-                                        gpointer         user_data);
-
 gboolean
 on_comp_event_box_enter_notify_event (GtkWidget       *widget,
                                         GdkEventCrossing *event,
@@ -853,6 +871,339 @@ on_stereo_event_box_enter_notify_event
                                         gpointer         user_data);
 
 gboolean
-on_stereo_balance_event_box_enter_notify_event (GtkWidget       *widget,
-                                            GdkEventCrossing *event,
-                                            gpointer         user_data);
+reset_range                            (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_at_label_1_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_re_label_1_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_th_label_1_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_ra_label_1_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_kn_label_1_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_ma_label_1_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_at_label_2_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_re_label_2_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_th_label_2_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_ra_label_2_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_kn_label_2_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_ma_label_2_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_at_label_3_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_re_label_3_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_th_label_3_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_ra_label_3_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_kn_label_3_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+gboolean
+on_comp_ma_label_3_event_box_button_press_event
+                                        (GtkWidget       *widget,
+                                        GdkEventButton  *event,
+                                        gpointer         user_data);
+
+void
+on_options1_activate                   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_eq_options1_activate                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_preferences1_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_spectrum_freq_spinbutton_value_changed
+                                        (GtkSpinButton   *spinbutton,
+                                        gpointer         user_data);
+
+void
+on_eq_options_close_clicked            (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_crossfade_spinbutton_value_changed  (GtkSpinButton   *spinbutton,
+                                        gpointer         user_data);
+
+void
+on_preferences_close_clicked           (GtkButton       *button,
+                                        gpointer         user_data);
+
+gboolean
+on_eq_options_event_box_enter_notify_event
+                                        (GtkWidget       *widget,
+                                        GdkEventCrossing *event,
+                                        gpointer         user_data);
+
+gboolean
+on_preferences_event_box_enter_notify_event
+                                        (GtkWidget       *widget,
+                                        GdkEventCrossing *event,
+                                        gpointer         user_data);
+
+gboolean
+on_text_focus_in_event                 (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data);
+
+gboolean
+on_text_focus_out_event                (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data);
+
+void
+on_low_active_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_low_bypass_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_low_mute_toggled                    (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_mid_active_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_mid_bypass_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_mid_mute_toggled                    (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_high_active_toggled                 (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_high_bypass_toggled                 (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_high_mute_toggled                   (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_hscale_1_l_value_changed            (GtkRange        *range,
+                                        gpointer         user_data);
+
+void
+on_hscale_1_l_realize                  (GtkWidget       *widget,
+                                        gpointer         user_data);
+
+void
+on_hscale_1_m_value_changed            (GtkRange        *range,
+                                        gpointer         user_data);
+
+void
+on_hscale_1_m_realize                  (GtkWidget       *widget,
+                                        gpointer         user_data);
+
+void
+on_hscale_1_h_value_changed            (GtkRange        *range,
+                                        gpointer         user_data);
+
+void
+on_hscale_1_h_realize                  (GtkWidget       *widget,
+                                        gpointer         user_data);
+
+void
+on_low_bypass_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_mid_bypass_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_high_bypass_toggled                 (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_solo_toggled                        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+gboolean
+on_eq_bypass_event_box_enter_notify_event
+                                        (GtkWidget       *widget,
+                                        GdkEventCrossing *event,
+                                        gpointer         user_data);
+
+void
+on_eq_bypass_toggled                   (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+gboolean
+on_band_button_event_box_enter_notify_event
+                                        (GtkWidget       *widget,
+                                        GdkEventCrossing *event,
+                                        gpointer         user_data);
+
+void
+on_limiter_bypass_toggled              (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+gboolean
+on_limiter_bypass_event_box_enter_notify_event
+                                        (GtkWidget       *widget,
+                                        GdkEventCrossing *event,
+                                        gpointer         user_data);
+
+void
+on_name_activate                       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_scene_name_entry_changed            (GtkEditable     *editable,
+                                        gpointer         user_data);
+
+void
+on_scene_name_cancel_clicked           (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_scene_name_ok_clicked               (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_low_band_compressor_color_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_mid_band_compressor_color_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_high_band_compressor_color_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_ganged_controls_color_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_parametric_handles_color_activate   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_hdeq_curve_color_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_hdeq_grid_color_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_hdeq_background_color_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_text_color_activate                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_meter_normal_color_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_meter_warning_color_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_meter_over_color_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_meter_peak_color_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_reset_all_colors1_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
