@@ -71,9 +71,9 @@
 #include <errno.h>
 #include <assert.h>
 #include <jack/jack.h>
-#include "config.h"
 
-#include "ringbuffer.h"
+#include "config.h"
+#include "ringbuffer.h"		/* uses <jack/ringbuffer.h>, if available */
 #include "process.h"
 #include "resource.h"
 #include "plugin.h"
@@ -81,17 +81,22 @@
 #include "transport.h"
 #include "jackstatus.h"
 #include "state.h"
+#include "spectrum.h"
 #include "debug.h"
+#include "help.h"
+#include "support.h"
 
-char *jamin_options = "dFf:n:hprTtvV"; /* valid JAMin options */
-char *pname;				/* `basename $0` */
-int dummy_mode = 0;			/* -d option */
-int all_errors_fatal = 0;		/* -F option */
-int show_help = 0;			/* -h option */
-int connect_ports = 1;			/* -p option */
-int trace_option = 0;			/* -T option */
-int thread_option = 1;			/* -t option */
-int debug_level = DBG_OFF;		/* -v option */
+char *jamin_options = "dFf:n:hprTtvVs:c:";    /* valid JAMin options */
+char *pname;				      /* `basename $0` */
+int dummy_mode = 0;			      /* -d option */
+int all_errors_fatal = 0;		      /* -F option */
+int show_help = 0;			      /* -h option */
+int connect_ports = 1;			      /* -p option */
+int trace_option = 0;			      /* -T option */
+int thread_option = 1;			      /* -t option */
+int debug_level = DBG_OFF;		      /* -v option */
+char session_file[PATH_MAX];		      /* -f option */
+static char *errstr;
 
 /*  Synchronization within the DSP engine is managed as a finite state
  *  machine.  These state transitions are the key to understanding
@@ -117,8 +122,8 @@ static pthread_cond_t run_dsp = PTHREAD_COND_INITIALIZER;
 static pthread_mutex_t lock_dsp = PTHREAD_MUTEX_INITIALIZER;
 
 #define NCHUNKS 4		/* number of DSP blocks in ringbuffer */
-static ringbuffer_t *in_rb[NCHANNELS];	/* input channel ring buffers */
-static ringbuffer_t *out_rb[NCHANNELS];	/* output channel ring buffers */
+static jack_ringbuffer_t *in_rb[NCHANNELS];  /* input channel buffers */
+static jack_ringbuffer_t *out_rb[NCHANNELS]; /* output channel buffers */
 
 /* JACK connection data */
 jack_status_t jst = {0};		/* current JACK status */
@@ -129,10 +134,10 @@ int nchannels = NCHANNELS;		/* actual number of channels */
 jack_port_t *input_ports[NCHANNELS+1] = {NULL};
 jack_port_t *output_ports[NCHANNELS+1] = {NULL};
 
-static char *in_names[NCHANNELS] = {"in_L", "in_R"};
-static char *out_names[NCHANNELS] = {"out_L", "out_R"};
-static char *iports[NCHANNELS] = {"", ""};
-static char *oports[NCHANNELS] = {"alsa_pcm:playback_1", "alsa_pcm:playback_2"};
+static const char *in_names[NCHANNELS] = {"in_L", "in_R"};
+static const char *out_names[NCHANNELS] = {"out_L", "out_R"};
+static const char *iports[NCHANNELS] = {NULL, NULL};
+static const char *oports[NCHANNELS] = {NULL, NULL};
 
 
 /****************  Low-level utility functions  ****************/
@@ -217,9 +222,9 @@ void io_errlog(int err, char *fmt, ...)
 
     IF_DEBUG(DBG_TERSE,
 	     io_trace("error %d: %s", err, buffer));
-    fprintf(stderr, "%s internal error %d: %s\n", PACKAGE, err, buffer);
+    fprintf(stderr, _("%s internal error %d: %s\n"), PACKAGE, err, buffer);
     if (all_errors_fatal) {
-	fprintf(stderr, " Terminating due to -F option.\n");
+	fprintf(stderr, _(" Terminating due to -F option.\n"));
 	abort();
     }
 }
@@ -317,11 +322,11 @@ int io_get_dsp_buffers(int nchannels,
 		       jack_default_audio_sample_t *out[NCHANNELS])
 {
     int chan;
-    ringbuffer_data_t io_vec[2];
+    jack_ringbuffer_data_t io_vec[2];
 
     for (chan = 0; chan < nchannels; chan++) {
-	if ((ringbuffer_read_space(in_rb[chan]) < dsp_block_bytes) ||
-	    (ringbuffer_write_space(out_rb[chan]) < dsp_block_bytes))
+	if ((jack_ringbuffer_read_space(in_rb[chan]) < dsp_block_bytes) ||
+	    (jack_ringbuffer_write_space(out_rb[chan]) < dsp_block_bytes))
 	    return 0;			/* not enough space */
 
 	/* Copy buffer pointers to in[] and out[].  If the ringbuffer
@@ -329,11 +334,11 @@ int io_get_dsp_buffers(int nchannels,
 	 * extend the interface to process_signal() to allow this
 	 * situation.  But, that's not implemented yet, hence the
 	 * asserts. */
-	ringbuffer_get_read_vector(in_rb[chan], io_vec);
+	jack_ringbuffer_get_read_vector(in_rb[chan], io_vec);
 	in[chan] = (jack_default_audio_sample_t *) io_vec[0].buf;
 	assert(io_vec[0].len >= dsp_block_bytes); /* must be contiguous */
 	    
-	ringbuffer_get_write_vector(out_rb[chan], io_vec);
+	jack_ringbuffer_get_write_vector(out_rb[chan], io_vec);
 	out[chan] = (jack_default_audio_sample_t *) io_vec[0].buf;
 	assert(io_vec[0].len >= dsp_block_bytes); /* must be contiguous */
     }
@@ -384,8 +389,8 @@ void *io_dsp_thread(void *arg)
 	     * space and queues the output for the JACK process
 	     * thread. */
 	    for (chan = 0; chan < nchannels; chan++) {
-		ringbuffer_write_advance(out_rb[chan], dsp_block_bytes);
-		ringbuffer_read_advance(in_rb[chan], dsp_block_bytes);
+		jack_ringbuffer_write_advance(out_rb[chan], dsp_block_bytes);
+		jack_ringbuffer_read_advance(in_rb[chan], dsp_block_bytes);
 	    }
 	    if (DSP_STATE_IS(DSP_STARTING))
 		io_new_state(DSP_RUNNING); /* output available */
@@ -452,7 +457,7 @@ int io_queue(jack_nframes_t nframes, int nchannels,
 
     /* queue JACK input buffers for DSP thread */
     for (chan = 0; chan < nchannels; chan++) {
-	count = ringbuffer_write(in_rb[chan], (void *) in[chan], nbytes);
+	count = jack_ringbuffer_write(in_rb[chan], (void *) in[chan], nbytes);
 	if (count != nbytes) {		/* buffer overflow? */
 
 	    /* This is a realtime bug.  We have input audio with no
@@ -468,12 +473,12 @@ int io_queue(jack_nframes_t nframes, int nchannels,
     } 
 
     /* if there is enough input, schedule the DSP thread */
-    if (ringbuffer_read_space(in_rb[0]) >= dsp_block_bytes)
+    if (jack_ringbuffer_read_space(in_rb[0]) >= dsp_block_bytes)
 	io_schedule();
 
     /* dequeue the next buffer that has been completed */
     for (chan = 0; chan < nchannels; chan++) {
-	count = ringbuffer_read(out_rb[chan], (void *) out[chan], nbytes);
+	count = jack_ringbuffer_read(out_rb[chan], (void *) out[chan], nbytes);
 	if (count != nbytes) {		/* not enough output? */
 
 	    /* this is only legit if we're just starting up */
@@ -641,9 +646,9 @@ void io_cleanup()
 	/* free the ring buffers */
 	for (chan = 0; chan < nchannels; chan++) {
 	    if (in_rb[chan])
-		ringbuffer_free(in_rb[chan]);
+		jack_ringbuffer_free(in_rb[chan]);
 	    if (out_rb[chan])
-		ringbuffer_free(out_rb[chan]);
+		jack_ringbuffer_free(out_rb[chan]);
 	}
     }
 
@@ -669,9 +674,11 @@ gboolean check_file (char *optarg)
 
   if ((fp = fopen (optarg, "r")) == NULL)
     {
-      fprintf (stderr, "\n\n");
-      perror (optarg);
-      fprintf (stderr, "Using default.\n\n");
+      errstr = g_strdup_printf (_("File %s : %s\nUsing default."), optarg, 
+                                strerror (errno));
+      fprintf (stderr, "%s\n", errstr);
+      message (GTK_MESSAGE_ERROR, errstr);
+      free (errstr);
       return (FALSE);
     }
 
@@ -691,9 +698,15 @@ gboolean check_file (char *optarg)
 void io_init(int argc, char *argv[])
 {
     int chan;
-    int opt;
+    int opt, spectrum_freq;
+    float crossfade_time;
     char *client_name = NULL;
 
+
+    spectrum_freq = 10;
+    crossfade_time = 1.0;
+
+
     /* basename $0 */
     pname = strrchr(argv[0], '/');
     if (pname == 0)
@@ -710,11 +723,23 @@ void io_init(int argc, char *argv[])
 	    all_errors_fatal = 1;
 	    break;
 	case 'f':
-            if (check_file (optarg)) s_set_filename(optarg);
+            if (check_file(optarg)) {
+		strncpy(session_file, optarg, sizeof(session_file));
+		s_set_filename(session_file);
+	    }
             break;
 	case 'n':			/* Set JACK client name */
 	    client_name = strdup(optarg);
 	    break;
+	case 's':			/* Set spectrum update frequency */
+	    sscanf (optarg, "%d", &spectrum_freq);
+            if (spectrum_freq < 0 || spectrum_freq > 10) spectrum_freq = 10;
+	    break;
+	case 'c':			/* Set crossfade time */
+	    sscanf (optarg, "%f", &crossfade_time);
+            if (crossfade_time < 0.0 || crossfade_time > 2.0) 
+              crossfade_time = 1.0;
+	    break;
 	case 'h':			/* show help */
 	    show_help = 1;
 	    break;
@@ -742,6 +767,11 @@ void io_init(int argc, char *argv[])
 	}
     }
 
+
+    set_spectrum_freq (spectrum_freq);
+    s_set_crossfade_time (crossfade_time);
+
+
     if (connect_ports) {
 
 	/* check for input and output port names of each channel */
@@ -758,12 +788,14 @@ void io_init(int argc, char *argv[])
 	show_help = 1;
 
     if (show_help) {
-	fprintf(stderr,
+	fprintf(stderr, _(
                 "Usage: %s [-%s] [inport1 inport2 [outport1 outport2]]\n"
                 "\nuser options:\n"
                 "\t-f file\tload session file on startup\n"
                 "\t-h\tshow this help\n"
                 "\t-n name\tset JACK client name\n"
+                "\t-s freq\tset spectrum update frequency\n"
+                "\t-c time\tset crossfade time\n"
                 "\t-r\tuse example GTK resource file\n"
                 "\t-p\tdon't automatically connect JACK output ports\n"
                 "\t-v\tverbose output (use -vv... for more detail)\n"
@@ -773,11 +805,12 @@ void io_init(int argc, char *argv[])
                 "\t-F\ttreat all errors as fatal\n"
                 "\t-T\tprint trace buffer\n"
                 "\t-t\tdon't start DSP thread\n"
-                "\n",
+                "\n"),
 		pname, jamin_options);
 	exit(1);
     }
 
+
     if (dummy_mode) {
 	io_new_state(DSP_STOPPED);
 	io_bufsize(1024, NULL);
@@ -790,12 +823,11 @@ void io_init(int argc, char *argv[])
     if (!client_name) {
 	client_name = strdup(PACKAGE);
     }
-    printf("Registering as %s\n", client_name);
 
     client = jack_client_new(client_name);
     if (client == 0) {
-	fprintf(stderr,
-		"%s: Cannot contact JACK server, is it running?\n", PACKAGE);
+	fprintf(stderr, _("%s: Cannot contact JACK server, is it running?\n"),
+		PACKAGE);
 	exit(2);
     }
 
@@ -877,6 +909,8 @@ int io_create_dsp_thread()
 	return 0;
     }
 
+#ifdef HAVE_POSIX_SCHED
+
     IF_DEBUG(DBG_TERSE, io_trace("first pthread_create() returns %d\n", rc));
 
     /* The following comment was copied from jack/libjack/client.c
@@ -933,10 +967,15 @@ int io_create_dsp_thread()
     rc = pthread_create(&dsp_thread, &attributes, io_dsp_thread, NULL);
     if (rc != 0) {
 	sched_setscheduler(0, policy, &my_param);
-	fprintf(stderr,
-		"%s: not permitted to create realtime DSP thread.\n"
-		"\tYou must run as root or use JACK capabilities.\n"
-		"\tContinuing operation, but with -t option.\n", PACKAGE);
+
+        errstr = g_strdup_printf (
+	    _("%s: not permitted to create realtime DSP thread.\n"
+	      "\tYou must run as root or use JACK capabilities.\n"
+	      "\tContinuing operation, but with -t option.\n"), PACKAGE);
+        fprintf (stderr, "%s\n", errstr);
+        message (GTK_MESSAGE_WARNING, errstr);
+        free (errstr);
+
 	IF_DEBUG(DBG_TERSE,
 		 io_trace("second pthread_create() returns %d\n", rc));
 	return rc;
@@ -946,6 +985,9 @@ int io_create_dsp_thread()
     sched_setscheduler(0, policy, &my_param);
     IF_DEBUG(DBG_TERSE, io_trace("DSP thread finally created"));
     return 0;
+#else /* !HAVE_POSIX_SCHED */
+    return rc;
+#endif /* HAVE_POSIX_SCHED */
 }
 
 
@@ -977,13 +1019,13 @@ void io_activate()
 			       JackPortIsOutput, 0);
 
 	if (input_ports[chan] == NULL || output_ports[chan] == NULL) {
-	    fprintf(stderr, "%s: Cannot register JACK ports.", PACKAGE);
+	    fprintf(stderr, _("%s: Cannot register JACK ports."), PACKAGE);
 	    exit(2);
 	}
     }
 
     if (jack_activate(client)) {
-	fprintf(stderr, "%s: Cannot activate JACK client.", PACKAGE);
+	fprintf(stderr, _("%s: Cannot activate JACK client."), PACKAGE);
 	exit(2);
     }
 
@@ -991,20 +1033,52 @@ void io_activate()
 
     /* connect any required JACK ports */
     if (connect_ports) {
+
+	const char **pports = NULL;
+
+	if (oports[0] == NULL) {	/* no output ports specified? */
+
+	    pports = jack_get_ports (client, ":playback", 
+				     JACK_DEFAULT_AUDIO_TYPE,
+				     JackPortIsPhysical|JackPortIsInput);
+	    if (pports) {
+		/* use first `nchannels' physical playback ports */
+		for (chan = 0; chan < nchannels && pports[chan]; chan++) {
+		    oports[chan] = pports[chan];
+		}
+	    } else {
+		errstr = g_strdup_printf(_("No physical playback ports found"));
+		fprintf (stderr, "%s\n", errstr);
+		message (GTK_MESSAGE_WARNING, errstr);
+		free (errstr);
+	    }
+	}
+
 	for (chan = 0; chan < nchannels; chan++) {
 	    if (iports[chan] && *iports[chan]) {
 		if (jack_connect(client, iports[chan],
-				 jack_port_name(input_ports[chan])))
-		    fprintf(stderr, "Cannot connect input port \"%s\"\n",
-			    iports[chan]);
+				 jack_port_name(input_ports[chan]))) {
+                    errstr = g_strdup_printf (
+                        _("Cannot connect input port \"%s\"\n"), iports[chan]);
+                    fprintf (stderr, "%s\n", errstr);
+                    message (GTK_MESSAGE_WARNING, errstr);
+                    free (errstr);
+		}
 	    }
 	    if (oports[chan] && *oports[chan]) {
 		if (jack_connect(client, jack_port_name(output_ports[chan]),
-				 oports[chan]))
-		    fprintf(stderr, "Cannot connect output port \"%s\"\n",
-			    oports[chan]);
+				 oports[chan])) {
+                    errstr = g_strdup_printf (
+                        _("Cannot connect output port \"%s\"\n"), oports[chan]);
+                    fprintf (stderr, "%s\n", errstr);
+                    message (GTK_MESSAGE_WARNING, errstr);
+                    free (errstr);
+		}
 	    }
 	}
+
+	if (pports)
+	    free(pports);
     }
 
     /* Allocate DSP engine ringbuffers.  Be careful to get the sizes
@@ -1014,9 +1088,9 @@ void io_activate()
      * touch all the pages in the buffers. */
     bufsize = dsp_block_bytes * NCHUNKS;
     for (chan = 0; chan < nchannels; chan++) {
-	in_rb[chan] = ringbuffer_create(bufsize);
+	in_rb[chan] = jack_ringbuffer_create(bufsize);
 	memset(in_rb[chan]->buf, 0, bufsize);
-	out_rb[chan] = ringbuffer_create(bufsize);
+	out_rb[chan] = jack_ringbuffer_create(bufsize);
 	memset(out_rb[chan]->buf, 0, bufsize);
     }
 
