@@ -11,7 +11,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: scenes.h,v 1.12 2004/01/08 15:59:33 jdepner Exp $
+ *  $Id: scenes.h,v 1.15 2004/05/07 16:24:48 theno23 Exp $
  */
 
 #ifndef SCENES_H
@@ -23,15 +23,12 @@
 #include <gtk/gtk.h>
 
 #include "state.h"
-
-
-#define NUM_SCENES 6
-
+#include "constants.h"
 
 void bind_scenes ();
 void select_scene (int number, int button);
 int get_current_scene ();
-void set_scene (int scene_num, gboolean morph);
+void set_scene (int scene_num);
 void clear_scene (int scene_num);
 const char *get_scene_name(int number);
 void set_scene_name (int number, const char *scene_name);
