@@ -11,7 +11,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  $Id: state.c,v 1.39 2004/01/10 13:09:19 theno23 Exp $
+ *  $Id: state.c,v 1.56 2004/05/06 22:48:16 jdepner Exp $
  */
 
 #include <stdio.h>
@@ -28,10 +28,14 @@
 
 #include "config.h"
 #include "main.h"
+#include "geq.h"
+#include "spectrum.h"
 #include "state.h"
 #include "process.h"
 #include "scenes.h"
 #include "hdeq.h"
+#include "compressor-ui.h"
+#include "help.h"
 
 /* A scene value to indicate that loading failed */
 #define LOAD_ERROR -2
@@ -46,6 +50,8 @@ static int              s_duration[S_SIZE];
 static int              s_changed[S_SIZE];
 static GtkAdjustment   *s_adjustment[S_SIZE];
 static s_callback_func  s_callback[S_SIZE];
+static char             *errstr = NULL;
+
 
 static s_state       *last_state = NULL;
 static int	      last_changed = S_NONE;
@@ -55,15 +61,34 @@ static GList         *undo_pos = NULL;
 
 static int suppress_feedback = 0;
 static int saved_scene;
+static float crossfade_time = 1.0;
 
 static void s_set_events(int id, float value);
-void set_EQ_curve_values ();
-void unset_scene_buttons ();
 void s_update_title();
 void s_history_add(const char *description);
+void s_save_global_int(xmlDocPtr doc, char *symbol, int value);
+void s_save_global_float(xmlDocPtr doc, char *symbol, float value);
+void s_save_global_gang(xmlDocPtr doc, char *p, int band, gboolean value);
 
 static const gchar *filename = NULL;
 
+/* global session parmaeters read from the XML file */
+
+typedef struct {
+    int scene;
+    int mode;
+    float freq;
+    float lgain;
+    float hgain;
+    float ct;
+    int gang_at[XO_BANDS];
+    int gang_re[XO_BANDS];
+    int gang_th[XO_BANDS];
+    int gang_ra[XO_BANDS];
+    int gang_kn[XO_BANDS];
+    int gang_ma[XO_BANDS];
+} xml_global_params;
+
 void state_init()
 {
     unsigned int i;
@@ -76,6 +101,21 @@ void state_init()
 	s_adjustment[i] = NULL;
 	s_callback[i] = NULL;
     }
+
+
+    /*  Defaults for notches.  */
+
+    s_value[S_NOTCH_Q(1)] = 5.0;
+    s_value[S_NOTCH_Q(2)] = 5.0;
+    s_value[S_NOTCH_Q(3)] = 5.0;
+
+    s_value[S_NOTCH_FREQ(0)] = hdeq_get_notch_default_freq (0);
+    s_value[S_NOTCH_FREQ(1)] = hdeq_get_notch_default_freq (1);
+    s_value[S_NOTCH_FREQ(2)] = hdeq_get_notch_default_freq (2);
+    s_value[S_NOTCH_FREQ(3)] = hdeq_get_notch_default_freq (3);
+    s_value[S_NOTCH_FREQ(4)] = hdeq_get_notch_default_freq (4);
+
+
     s_history_add("Initial state");
 }
 
@@ -230,480 +270,7 @@ void s_history_add_state(s_state state)
     last_state = ns;
 }
 
-void s_undo() 
-{
-    GList *undo_next;
-    int       scene, crc[2];
-    s_state   *st[2];
-
-    unsigned int compute_state_crc (s_state *state);
-
-
-    if (!undo_pos) {
-	return;
-    }
-    undo_next = g_list_previous(undo_pos);
-    if (!undo_next) {
-	return;
-    }
-    undo_pos = undo_next;
-    s_restore_state((s_state *)undo_pos->data);
-
-    scene = get_previous_scene_num ();
-    if (scene >= 0)
-      {
-        st[0] = get_scene (scene);
-        st[1] = (s_state *) undo_pos->data;
-
-        crc[0] = compute_state_crc (st[0]);
-        crc[1] = compute_state_crc (st[1]);
-
-        if (crc[0] == crc[1]) set_scene_button (scene);
-      }
-
-    set_EQ_curve_values ();
-    last_changed = S_LOAD;
-}
-
-void s_redo() 
-{
-    gboolean  restore;
-    int       scene, crc[2];
-    s_state   *st[2];
-
-    unsigned int compute_state_crc (s_state *state);
-
-
-    restore = FALSE;
-    if (undo_pos) {
-	if (undo_pos->next) {
-	    undo_pos = g_list_next(undo_pos);
-            restore = TRUE;
-	}
-    } else {
-	undo_pos = history;
-	undo_pos = g_list_next(undo_pos);
-        restore = TRUE;
-    }
-
-    if (restore)
-      {
-        s_restore_state((s_state *)undo_pos->data);
-
-        scene = get_previous_scene_num ();
-        if (scene >= 0)
-          {
-            st[0] = get_scene (scene);
-            st[1] = (s_state *) undo_pos->data;
-
-            crc[0] = compute_state_crc (st[0]);
-            crc[1] = compute_state_crc (st[1]);
-
-            if (crc[0] == crc[1]) 
-              {
-                set_scene_button (scene);
-              }
-            else
-              {
-                set_scene_warning_button (scene);
-              }
-          }
-        set_EQ_curve_values ();
-      }
-}
-
-void s_crossfade_to_state(s_state *state, float time)
-{
-    int i, duration;
-
-    /* printf("restore %s\n", state->description); */
-    duration = (int)(sample_rate * time);
-    suppress_feedback++;
-    for (i=0; i<S_SIZE; i++) {
-	/* set the target and duration for crosssfade, but set the controls to
-	 * the endpoint */
-	s_target[i] = state->value[i];
-	s_duration[i] = duration;
-	//s_set_events(i, state->value[i]);
-    }
-    suppress_feedback--;
-}
-
-void s_restore_state(s_state *state)
-{
-    s_crossfade_to_state (state, 0.003f);
-}
-
-static void s_set_events(int id, float value)
-{
-    if (s_callback[id]) {
-	(*s_callback[id])(id, value);
-    }
-    if (s_adjustment[id]) {
-	gtk_adjustment_set_value(s_adjustment[id], value);
-    }
-}
-
-void s_set_description(int id, const char *desc)
-{
-    if (last_changed != id) {
-	s_history_add(desc);
-    }
-    last_changed = id;
-}
-
-void s_save_session_from_ui (GtkWidget *w, gpointer user_data)
-{
-    GtkFileSelection *file_selector = (GtkFileSelection *) user_data;
-
-    s_save_session(gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector)));
-}
-    
-void s_save_session (const char *fname)
-{
-    xmlDocPtr doc;
-    xmlNodePtr rootnode, node, sc_node;
-    unsigned int i, j;
-    int curr_scene;
-    char tmp[256];
-
-    /* Check to see if we have been passed a filename, if not fall back to
-     * previous one */
-    if (fname) {
-	filename = fname;
-	s_update_title();
-    }
-    if (!filename) {
-	fprintf(stderr, "No filename found at %s:%d, not saving\n", __FILE__,
-		__LINE__);
-    }
-
-    /*  Need to save this scene number.  */
-    curr_scene = get_current_scene ();
-
-    xmlSetCompressMode(5);
-    doc = xmlNewDoc("1.0");
-    rootnode = xmlNewDocRawNode(doc, NULL, "jam-param-list", NULL);
-    xmlSetProp(rootnode, "version", VERSION);
-    xmlDocSetRootElement(doc, rootnode);
-    node = xmlNewText("\n");
-    xmlAddChild(rootnode, node);
-
-    /* Save current active state */
-    for (i=0; i<S_SIZE; i++) {
-	node = xmlNewDocRawNode(doc, NULL, "parameter", NULL);
-	snprintf(tmp, 255, "%g", s_value[i]);
-	xmlSetProp(node, "name", s_symbol[i]);
-	xmlSetProp(node, "value", tmp);
-	xmlAddChild(rootnode, node);
-	node = xmlNewText("\n");
-	xmlAddChild(rootnode, node);
-    }
-
-    /* Save scenes */
-    for (j=0; j<NUM_SCENES; j++) {
-	s_state *st = get_scene(j);
-	sc_node = xmlNewDocRawNode(doc, NULL, "scene", NULL);
-	snprintf(tmp, 255, "%d", j);
-	xmlSetProp(sc_node, "number", tmp);
-	if (!st) {
-	    xmlAddChild(rootnode, sc_node);
-	    node = xmlNewText("\n");
-	    xmlAddChild(rootnode, node);
-	    continue;
-	}
-	xmlSetProp(sc_node, "name", get_scene_name(j));
-	if (curr_scene == j) {
-	    xmlSetProp(sc_node, "active", "true");
-	    xmlSetProp(sc_node, "changed", "false");
-	} else if (curr_scene == changed_scene_no(j)) {
-	    xmlSetProp(sc_node, "active", "true");
-	    xmlSetProp(sc_node, "changed", "true");
-	}
-	node = xmlNewText("\n");
-	xmlAddChild(sc_node, node);
-	xmlAddChild(rootnode, sc_node);
-	node = xmlNewText("\n");
-	xmlAddChild(rootnode, node);
-
-	for (i=0; i<S_SIZE; i++) {
-	    node = xmlNewDocRawNode(doc, NULL, "parameter", NULL);
-	    snprintf(tmp, 255, "%g", st->value[i]);
-	    xmlSetProp(node, "name", s_symbol[i]);
-	    xmlSetProp(node, "value", tmp);
-	    xmlAddChild(sc_node, node);
-	    node = xmlNewText("\n");
-	    xmlAddChild(sc_node, node);
-	}
-    }
-    xmlSaveFile(filename, doc);
-    xmlFreeDoc(doc);
-}
-
-/* declare SAX handlers */
-void s_startElement(void *user_data, const xmlChar *name,
-                    const xmlChar **attrs);
-
-static void s_warning(void *user_data, const char *msg, ...);
-
-static void s_error(void *user_data, const char *msg, ...);
-
-
-void s_load_session_from_ui (GtkWidget *w, gpointer user_data)
-{
-    GtkFileSelection *file_selector = (GtkFileSelection *) user_data;
-
-    s_load_session(gtk_file_selection_get_filename (GTK_FILE_SELECTION
-                                                (file_selector)));
-}
-    
-void s_load_session (const char *fname)
-{
-    xmlSAXHandlerPtr handler;
-    int scene = -1;
-    int fd;
-
-    saved_scene = -1;
-    unset_scene_buttons ();
-
-    if (fname) {
-	filename = fname;
-    }
-    if (filename == NULL) {
-	filename = default_session;
-    }
-
-    /* Check file exists */
-    if ((fd = open(filename, O_RDONLY)) >= 0) {
-	close(fd);
-    } else {
-	char *errstr = NULL;
-
-	errstr = g_strdup_printf("Error opening '%s'", filename);
-	perror(errstr);
-	free(errstr);
-
-	if (fname == NULL) {
-	    exit(1);
-	}
-    }
-	
-    s_update_title();
-
-    handler = calloc(1, sizeof(xmlSAXHandler));
-    handler->startElement = s_startElement;
-    handler->warning = s_warning;
-    handler->error = s_error;
-
-    xmlSAXUserParseFile(handler, &scene, filename);
-
-    if (scene == LOAD_ERROR) {
-	fprintf(stderr, "Loading failed\n");
-	return;
-    }
-
-    s_history_add(g_strdup_printf("Load %s", filename));
-    last_changed = S_LOAD;
-    free(handler);
-
-
-    /*  This is the active scene.  */
-
-    if (saved_scene < 100)
-      {
-        set_scene (saved_scene, FALSE);
-      }
-    else
-      {
-        set_num_scene_warning_button(saved_scene);
-      }
-
-    if (!fname) {
-	filename = NULL;
-    }
-
-    hdeq_set_xover ();
-    set_EQ_curve_values ();
-
-    s_clear_history();
-}
-
-void s_startElement(void *user_data, const xmlChar *name, const xmlChar **attrs)
-{
-    const xmlChar **p;
-    unsigned int i, found = 0;
-    const char *symbol = NULL, *value = NULL;
-    int *scene = (int *)user_data;
-    int active = 0;
-    int changed = 0;
-
-    if (!strcmp(name, "jam-param-list")) {
-	return;
-    }
-
-    if (!strcmp(name, "scene")) {
-	const char *sname = NULL;
-
-	for (p=attrs; p && *p; p+=2) {
-	    if (!strcmp(*p, "name")) {
-		sname = *(p+1);
-	    } else if (!strcmp(*p, "number")) {
-		*scene = atoi(*(p+1));
-	    } else if (!strcmp(*p, "active") && !strcmp(*(p+1), "true")) {
-		active = 1;
-	    } else if (!strcmp(*p, "changed") && !strcmp(*(p+1), "true")) {
-		changed = 1;
-	    }
-	}
-
-
-        /*  Set the active scene number to be set after parsing all of the
-            scenes in the XML file.  */
-
-	if (active) {
-            saved_scene = *scene;
-	    set_scene(*scene, FALSE);
-	}
-	if (changed) {
-            saved_scene = *scene + 100;
-	    set_num_scene_warning_button(changed_scene_no(*scene));
-	}
-
-	if (sname && *scene > -1) {
-	    set_scene(*scene, FALSE);
-	    set_scene_name(*scene, sname);
-	}
-
-	return;
-    }
-
-    /* Check its a parameter element */
-    if (strcmp(name, "parameter")) {
-	fprintf(stderr, "Unhandled element: %s\n", name);
-	return;
-    }
-
-    /* Find the name and value attributes */
-    for (p=attrs; p && *p; p+=2) {
-	if (!strcmp(*p, "name")) {
-	    symbol = *(p+1);
-	} else if (!strcmp(*p, "value")) {
-	    value = *(p+1);
-	}
-    }
-
-    /* Find the matching symbol, this is horribly inefficient */
-    for (i=0; i<S_SIZE && !found; i++) {
-	if (!strcmp(symbol, s_symbol[i])) {
-	    if (*scene == -1) {
-		s_value[i] = atof(value);
-		suppress_feedback++;
-		s_set_events(i, s_value[i]);
-		suppress_feedback--;
-	    } else {
-		s_state *st = get_scene(*scene);
-		if (st) {
-		    st->value[i] = atof(value);
-		} else {
-		    fprintf(stderr, "Bad scene number %d\n", *scene);
-		}
-	    }
-	    //printf("load %s = %g\n", symbol, s_value[i]);
-	    found = 1;
-	    break;
-	}
-    }
-    if (!found) {
-	fprintf(stderr, "Unknown symbol: %s\n", *p);
-    }
-}
-
-static void s_warning(void *user_data, const char *msg, ...)
-{
-    va_list args;
-    char *fmt;
-
-    va_start(args, msg);
-    fmt = g_strdup_printf("XML parser warning: %s", msg);
-    vfprintf(stderr, fmt, args);
-    free(fmt);
-    va_end(args);
-}
-
-static void s_error(void *user_data, const char *msg, ...)
-{
-    va_list args;
-    char *fmt;
-
-    va_start(args, msg);
-    fmt = g_strdup_printf("XML parser error: %s", msg);
-    vfprintf(stderr, fmt, args);
-    free(fmt);
-    va_end(args);
-    *((int *)user_data) = LOAD_ERROR;
-}
-
-void s_crossfade(const int nframes)
-{
-    unsigned int i;
-
-    /*  We don't want warning flags when we're cross fading.  */
-
-    for (i=0; i<S_SIZE; i++) {
-        if (s_duration[i] != 0) {
-/* debug crap if (i == S_IN_GAIN) printf("%d\t%f\n", s_duration[i], s_value[i]); */
-            s_duration[i] -= nframes;
-            if (s_duration[i] > nframes) {
-                s_value[i] += ((float)nframes / (float)s_duration[i]) *
-                                (s_target[i] - s_value[i]);
-            } else {
-                s_value[i] = s_target[i];
-                s_duration[i] = 0;
-            }
-	    s_changed[i] = 1;
-        }
-    }
-}
-
-void s_crossfade_ui()
-{
-    unsigned int i;
-
-    suppress_feedback++;
-    for (i=0; i<S_SIZE; i++) {
-	if (s_changed[i]) {
-	    s_changed[i] = 0;
-	    s_set_events(i, s_value[i]);
-	}
-    }
-    suppress_feedback--;
-}
-
-int s_have_filename()
-{
-    return (filename != NULL);
-}
-
-void s_update_title()
-{
-    char *title; 
-   char *base;
-    char *tmp;
-
-    tmp = strdup(filename);
-    base = basename(tmp);
-    title = g_strdup_printf(PACKAGE " - %s - " VERSION, base);
-    free(tmp);
-    gtk_window_set_title ((GtkWindow *) main_window, title);
-    free(title);
-}
-
-void s_set_filename(const char *fname)
-{
-    filename = fname;
-}
-
-unsigned int compute_state_crc (s_state *state)
+static unsigned int compute_state_crc (s_state *state)
 {
     unsigned int        checksum, i;
     unsigned char       *buf;
@@ -766,4 +333,697 @@ unsigned int compute_state_crc (s_state *state)
     return (checksum);
 }
 
+void s_undo() 
+{
+    GList *undo_next;
+    int       scene, crc[2];
+    s_state   *st[2];
+
+
+    if (!undo_pos) {
+	return;
+    }
+    undo_next = g_list_previous(undo_pos);
+    if (!undo_next) {
+	return;
+    }
+    undo_pos = undo_next;
+    s_restore_state((s_state *)undo_pos->data);
+
+    scene = get_previous_scene_num ();
+    if (scene >= 0)
+      {
+        st[0] = get_scene (scene);
+        st[1] = (s_state *) undo_pos->data;
+
+        crc[0] = compute_state_crc (st[0]);
+        crc[1] = compute_state_crc (st[1]);
+
+        if (crc[0] == crc[1]) set_scene_button (scene);
+      }
+
+    set_EQ_curve_values (0, 0.0);
+    last_changed = S_LOAD;
+}
+
+void s_redo() 
+{
+    gboolean  restore;
+    int       scene, crc[2];
+    s_state   *st[2];
+
+
+    restore = FALSE;
+    if (undo_pos) {
+	if (undo_pos->next) {
+	    undo_pos = g_list_next(undo_pos);
+            restore = TRUE;
+	}
+    } else {
+	undo_pos = history;
+	undo_pos = g_list_next(undo_pos);
+        restore = TRUE;
+    }
+
+    if (restore)
+      {
+        s_restore_state((s_state *)undo_pos->data);
+
+        scene = get_previous_scene_num ();
+        if (scene >= 0)
+          {
+            st[0] = get_scene (scene);
+            st[1] = (s_state *) undo_pos->data;
+
+            crc[0] = compute_state_crc (st[0]);
+            crc[1] = compute_state_crc (st[1]);
+
+            if (crc[0] == crc[1]) 
+              {
+                set_scene_button (scene);
+              }
+            else
+              {
+                set_scene_warning_button (scene);
+              }
+          }
+        set_EQ_curve_values (0, 0.0);
+      }
+}
+
+
+/*  Negative time will use the default "crossfade time" which may come from the
+    command line -c option.  */
+
+void s_crossfade_to_state(s_state *state, float time)
+{
+    int i, duration, milliseconds;
+
+    if (time < 0.0) time = crossfade_time;
+
+
+    suspend_ganging ();
+    milliseconds = time * 1000.0 + 50;
+    g_timeout_add (milliseconds, unsuspend_ganging, NULL);
+
+
+    /* printf("restore %s\n", state->description); */
+    duration = (int)(sample_rate * time);
+    suppress_feedback++;
+    for (i=0; i<S_SIZE; i++) {
+	/* set the target and duration for crosssfade, but set the controls to
+	 * the endpoint */
+	s_target[i] = state->value[i];
+	s_duration[i] = duration;
+	//s_set_events(i, state->value[i]);
+    }
+    suppress_feedback--;
+}
+
+void s_restore_state(s_state *state)
+{
+    s_crossfade_to_state (state, 0.003);
+}
+
+static void s_set_events(int id, float value)
+{
+    if (s_callback[id]) {
+	(*s_callback[id])(id, value);
+    }
+    if (s_adjustment[id]) {
+	gtk_adjustment_set_value(s_adjustment[id], value);
+    }
+}
+
+void s_set_description(int id, const char *desc)
+{
+    if (last_changed != id) {
+	s_history_add(desc);
+    }
+    last_changed = id;
+}
+
+void s_save_session_from_ui (GtkWidget *w, gpointer user_data)
+{
+    GtkFileSelection *file_selector = (GtkFileSelection *) user_data;
+
+    s_save_session(gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector)));
+}
+    
+void s_save_session (const char *fname)
+{
+    xmlDocPtr doc;
+    xmlNodePtr rootnode, node, sc_node;
+    unsigned int i, j;
+    int curr_scene;
+    char tmp[256];
+
+
+    /* Check to see if we have been passed a filename, if not fall back to
+     * previous one */
+
+    if (fname) {
+	filename = fname;
+	s_update_title();
+    }
+    if (!filename) {
+	errstr = g_strdup_printf("No filename found at %s:%d, not saving\n",
+                                 __FILE__, __LINE__);
+	message (GTK_MESSAGE_WARNING, errstr);
+	free(errstr);
+    }
+
+
+    /*  Need to save this scene number.  */
+
+    curr_scene = get_current_scene ();
+
+    xmlSetCompressMode(5);
+    doc = xmlNewDoc("1.0");
+    rootnode = xmlNewDocRawNode(doc, NULL, "jam-param-list", NULL);
+    xmlSetProp(rootnode, "version", VERSION);
+    xmlDocSetRootElement(doc, rootnode);
+    node = xmlNewText("\n");
+    xmlAddChild(rootnode, node);
+
+
+    s_save_global_int(doc, "mode", process_get_spec_mode());
+    s_save_global_int(doc, "freq", get_spectrum_freq());
+    s_save_global_float(doc, "ct", crossfade_time);
+    s_save_global_float(doc, "lgain", hdeq_get_lower_gain());
+    s_save_global_float(doc, "hgain", hdeq_get_upper_gain());
+
+
+    /* record the current gang state of the compressor controls */
+
+    for (i = 0 ; i < XO_BANDS ; i++) {
+        s_save_global_gang(doc, "at", i, comp_at_ganged(i));
+        s_save_global_gang(doc, "re", i, comp_re_ganged(i));
+        s_save_global_gang(doc, "th", i, comp_th_ganged(i));
+        s_save_global_gang(doc, "ra", i, comp_ra_ganged(i));
+        s_save_global_gang(doc, "kn", i, comp_kn_ganged(i));
+        s_save_global_gang(doc, "ma", i, comp_ma_ganged(i));
+    }
+
+
+    /* Save current active state */
+
+    for (i=0; i<S_SIZE; i++) {
+	node = xmlNewDocRawNode(doc, NULL, "parameter", NULL);
+	snprintf(tmp, 255, "%g", s_value[i]);
+	xmlSetProp(node, "name", s_symbol[i]);
+	xmlSetProp(node, "value", tmp);
+	xmlAddChild(rootnode, node);
+	node = xmlNewText("\n");
+	xmlAddChild(rootnode, node);
+    }
+
+
+    /* Save scenes */
+
+    for (j=0; j<NUM_SCENES; j++) {
+	s_state *st = get_scene(j);
+	sc_node = xmlNewDocRawNode(doc, NULL, "scene", NULL);
+	snprintf(tmp, 255, "%d", j);
+	xmlSetProp(sc_node, "number", tmp);
+	if (!st) {
+	    xmlAddChild(rootnode, sc_node);
+	    node = xmlNewText("\n");
+	    xmlAddChild(rootnode, node);
+	    continue;
+	}
+	xmlSetProp(sc_node, "name", get_scene_name(j));
+	if (curr_scene == j) {
+	    xmlSetProp(sc_node, "active", "true");
+	    xmlSetProp(sc_node, "changed", "false");
+	} else if (curr_scene == changed_scene_no(j)) {
+	    xmlSetProp(sc_node, "active", "true");
+	    xmlSetProp(sc_node, "changed", "true");
+	}
+	node = xmlNewText("\n");
+	xmlAddChild(sc_node, node);
+	xmlAddChild(rootnode, sc_node);
+	node = xmlNewText("\n");
+	xmlAddChild(rootnode, node);
+
+	for (i=0; i<S_SIZE; i++) {
+	    node = xmlNewDocRawNode(doc, NULL, "parameter", NULL);
+	    snprintf(tmp, 255, "%g", st->value[i]);
+	    xmlSetProp(node, "name", s_symbol[i]);
+	    xmlSetProp(node, "value", tmp);
+	    xmlAddChild(sc_node, node);
+	    node = xmlNewText("\n");
+	    xmlAddChild(sc_node, node);
+	}
+    }
+    xmlSaveFile(filename, doc);
+    xmlFreeDoc(doc);
+}
+
+/* declare SAX handlers */
+void s_startElement(void *user_data, const xmlChar *name,
+                    const xmlChar **attrs);
+
+static void s_warning(void *user_data, const char *msg, ...);
+
+static void s_error(void *user_data, const char *msg, ...);
+
+
+void s_load_session_from_ui (GtkWidget *w, gpointer user_data)
+{
+    GtkFileSelection *file_selector = (GtkFileSelection *) user_data;
+
+    s_load_session(gtk_file_selection_get_filename (GTK_FILE_SELECTION
+                                                (file_selector)));
+}
+    
+void s_load_session (const char *fname)
+{
+    xmlSAXHandlerPtr handler;
+    int scene = -1;
+    int fd;
+    int i;
+    xml_global_params gp;
+
+    saved_scene = -1;
+    unset_scene_buttons ();
+
+    if (fname) {
+	filename = fname;
+    }
+    if (filename == NULL) {
+	filename = default_session;
+    }
+
+    /* Check to see if file is readable */
+    if ((fd = open(filename, O_RDONLY)) >= 0) {
+	close(fd);
+    } else {
+	errstr = g_strdup_printf("Error opening '%s'", filename);
+	message (GTK_MESSAGE_WARNING, errstr);
+        perror(errstr);
+	free(errstr);
+
+	if (fname == NULL) {
+	    exit(1);
+	}
+    }
+
+    s_update_title();
+
+    handler = calloc(1, sizeof(xmlSAXHandler));
+    handler->startElement = s_startElement;
+    handler->warning = s_warning;
+    handler->error = s_error;
+
+    /* set the gp struct to some sensible defualts incase values aren't set in
+     * the XML file */
+    gp.scene = scene;
+    gp.mode = SPEC_POST_EQ;
+    gp.freq = 10;
+    gp.lgain = -12.0;
+    gp.hgain = 12.0;
+    gp.ct = 1.0;
+    for (i = 0 ; i < XO_BANDS ; i++) {
+        gp.gang_at[i] = FALSE;
+        gp.gang_re[i] = FALSE;
+        gp.gang_th[i] = FALSE;
+        gp.gang_ra[i] = FALSE;
+        gp.gang_kn[i] = FALSE;
+        gp.gang_ma[i] = FALSE;
+    }
+
+    /* run the SAX parser */    
+    xmlSAXUserParseFile(handler, &gp, filename);
+
+    if (gp.scene == LOAD_ERROR) {
+	errstr = g_strdup_printf("Loading file '%s' failed", filename);
+	message (GTK_MESSAGE_WARNING, errstr);
+        perror(errstr);
+	free(errstr);
+	return;
+    }
+
+    s_history_add(g_strdup_printf("Load %s", filename));
+    last_changed = S_LOAD;
+    free(handler);
+
+    /* global params are read into the gp struct, and set in the UI code from
+     * here */
+
+    process_set_spec_mode (gp.mode);
+    set_spectrum_freq (gp.freq);
+    hdeq_set_upper_gain (gp.hgain);
+    geq_set_range (geq_get_adjustment(0)->lower, gp.hgain);
+    hdeq_set_lower_gain (gp.lgain);
+    geq_set_range (gp.lgain, geq_get_adjustment(0)->upper);
+    s_set_crossfade_time (gp.ct);
+
+
+    /*  This is the active scene.  */
+
+    if (saved_scene < 100)
+      {
+        set_scene (saved_scene);
+      }
+    else
+      {
+        set_num_scene_warning_button(saved_scene);
+      }
+
+    if (!fname) {
+	filename = NULL;
+    }
+
+
+    /*  Set the ganging after the scene otherwise it will try to gang move
+        the scene values.  */
+
+    for (i = 0 ; i < XO_BANDS ; i++) {
+        if (gp.gang_at[i]) comp_gang_at (i);
+        if (gp.gang_re[i]) comp_gang_re (i);
+        if (gp.gang_th[i]) comp_gang_th (i);
+        if (gp.gang_ra[i]) comp_gang_ra (i);
+        if (gp.gang_kn[i]) comp_gang_kn (i);
+        if (gp.gang_ma[i]) comp_gang_ma (i);
+    }
+
+
+    hdeq_set_xover ();
+    set_EQ_curve_values (0, 0.0);
+
+    s_clear_history();
+}
+
+void s_startElement(void *user_data, const xmlChar *name, const xmlChar **attrs)
+{
+    const xmlChar **p;
+    unsigned int i, found = 0;
+    const char *symbol = NULL, *value = NULL, *index = NULL;
+    xml_global_params *gp = user_data;
+    int active = 0;
+    int changed = 0;
+
+    if (!strcmp(name, "jam-param-list")) {
+	return;
+    }
+
+    if (!strcmp(name, "scene")) {
+	const char *sname = NULL;
+
+	for (p=attrs; p && *p; p+=2) {
+	    if (!strcmp(*p, "name")) {
+		sname = *(p+1);
+	    } else if (!strcmp(*p, "number")) {
+		gp->scene = atoi(*(p+1));
+	    } else if (!strcmp(*p, "active") && !strcmp(*(p+1), "true")) {
+		active = 1;
+	    } else if (!strcmp(*p, "changed") && !strcmp(*(p+1), "true")) {
+		changed = 1;
+	    }
+	}
+
+
+        /*  Set the active scene number to be set after parsing all of the
+            scenes in the XML file.  */
+
+	if (active) {
+            saved_scene = gp->scene;
+	    set_scene(gp->scene);
+	}
+	if (changed) {
+            saved_scene = gp->scene + 100;
+	    set_num_scene_warning_button(changed_scene_no(gp->scene));
+	}
+
+	if (sname && gp->scene > -1) {
+	    set_scene(gp->scene);
+	    set_scene_name(gp->scene, sname);
+	}
+
+	return;
+    }
+
+    /* if its a global setting */
+    if (!strcmp(name, "global")) {
+	/* find the name, value and index attributes */
+	for (p=attrs; p && *p; p+=2) {
+	    if (!strcmp(*p, "name")) {
+		symbol = *(p+1);
+	    } else if (!strcmp(*p, "value")) {
+		value = *(p+1);
+	    } else if (!strcmp(*p, "index")) {
+		index = *(p+1);
+	    }
+	}
+
+	if (!strcmp(symbol, "mode")) {
+	    gp->mode = atoi(value);
+	} else if (!strcmp(symbol, "freq")) {
+	    gp->freq = atof(value);
+	} else if (!strcmp(symbol, "lgain")) {
+	    gp->lgain = atof(value);
+	} else if (!strcmp(symbol, "hgain")) {
+	    gp->hgain = atof(value);
+	} else if (!strcmp(symbol, "ct")) {
+	    gp->ct = atof(value);
+	} else if ((const char *)strstr(symbol, "gang_") == symbol) {
+	    int ind = index ? atoi(index) : -1;
+	    int val = atoi(value);
+
+	    if (ind >= 0 && ind < XO_BANDS) {
+		if (!strcmp(symbol, "gang_at")) {
+		    gp->gang_at[ind] = val;
+		} else if (!strcmp(symbol, "gang_re")) {
+		    gp->gang_re[ind] = val;
+		} else if (!strcmp(symbol, "gang_th")) {
+		    gp->gang_th[ind] = val;
+		} else if (!strcmp(symbol, "gang_ra")) {
+		    gp->gang_ra[ind] = val;
+		} else if (!strcmp(symbol, "gang_kn")) {
+		    gp->gang_kn[ind] = val;
+		} else if (!strcmp(symbol, "gang_ma")) {
+		    gp->gang_ma[ind] = val;
+		} else {
+		    errstr = g_strdup_printf("Unhandled gang: %s\n", symbol);
+		    message (GTK_MESSAGE_WARNING, errstr);
+		    free(errstr);
+		}
+	    } else {
+		errstr = g_strdup_printf("Unhandled index: %s\n", index);
+		message (GTK_MESSAGE_WARNING, errstr);
+		free(errstr);
+	    }
+	} else {
+	    errstr = g_strdup_printf("Unhandled global paramter: %s\n", symbol);
+	    message (GTK_MESSAGE_WARNING, errstr);
+	    free(errstr);
+	}
+
+	return;
+    }
+
+    /* Check its a parameter element */
+    if (strcmp(name, "parameter")) {
+	errstr = g_strdup_printf("Unhandled element: %s\n", name);
+	message (GTK_MESSAGE_WARNING, errstr);
+	free(errstr);
+    }
+
+    /* Find the name and value attributes */
+    for (p=attrs; p && *p; p+=2) {
+	if (!strcmp(*p, "name")) {
+	    symbol = *(p+1);
+	} else if (!strcmp(*p, "value")) {
+	    value = *(p+1);
+	}
+    }
+
+    /* Find the matching symbol, this is horribly inefficient */
+    for (i=0; i<S_SIZE && !found; i++) {
+	if (!strcmp(symbol, s_symbol[i])) {
+	    if (gp->scene == -1) {
+		s_value[i] = atof(value);
+		suppress_feedback++;
+		s_set_events(i, s_value[i]);
+		suppress_feedback--;
+	    } else {
+		s_state *st = get_scene(gp->scene);
+		if (st) {
+		    st->value[i] = atof(value);
+		} else {
+                  errstr = g_strdup_printf("Bad scene number %d\n", gp->scene);
+                  message (GTK_MESSAGE_WARNING, errstr);
+                  free(errstr);
+		}
+	    }
+	    //printf("load %s = %g\n", symbol, s_value[i]);
+	    found = 1;
+	    break;
+	}
+    }
+    if (!found) {
+
+      /*  We need to disregard stereo-balance settings as these have been removed
+          but may still be in old .jam files.  */
+
+      if (!strstr (symbol ,"stereo-balance"))
+        {
+	  errstr = g_strdup_printf("Unknown symbol: %s in element %s\n",
+				   symbol, name);
+          message (GTK_MESSAGE_WARNING, errstr);
+          free(errstr);
+        }
+    }
+}
+
+static void s_warning(void *user_data, const char *msg, ...)
+{
+    va_list args;
+    char *fmt;
+
+    va_start(args, msg);
+    fmt = g_strdup_printf("XML parser warning: %s", msg);
+    vfprintf(stderr, fmt, args);
+    free(fmt);
+    va_end(args);
+}
+
+static void s_error(void *user_data, const char *msg, ...)
+{
+    va_list args;
+    char *fmt;
+
+    va_start(args, msg);
+    fmt = g_strdup_printf("XML parser error: %s", msg);
+    vfprintf(stderr, fmt, args);
+    free(fmt);
+    va_end(args);
+    *((int *)user_data) = LOAD_ERROR;
+}
+
+void s_crossfade(const int nframes)
+{
+    unsigned int i;
+
+
+    for (i=0; i<S_SIZE; i++) {
+        if (s_duration[i] != 0) {
+/* debug crap if (i == S_IN_GAIN) printf("%d\t%f\n", s_duration[i], s_value[i]); */
+            s_duration[i] -= nframes;
+            if (s_duration[i] > nframes) {
+                s_value[i] += ((float)nframes / (float)s_duration[i]) *
+                                (s_target[i] - s_value[i]);
+            } else {
+                s_value[i] = s_target[i];
+                s_duration[i] = 0;
+            }
+	    s_changed[i] = 1;
+        }
+    }
+}
+
+void s_crossfade_ui()
+{
+    unsigned int i;
+
+
+    suppress_feedback++;
+    for (i=0; i<S_SIZE; i++) {
+	if (s_changed[i]) {
+	    s_changed[i] = 0;
+	    s_set_events(i, s_value[i]);
+	}
+    }
+    suppress_feedback--;
+}
+
+int s_have_filename()
+{
+    return (filename != NULL);
+}
+
+char *s_get_filename()
+{
+    return ((char *) filename);
+}
+
+void s_update_title()
+{
+    char *title; 
+   char *base;
+    char *tmp;
+
+    tmp = strdup(filename);
+    base = basename(tmp);
+    title = g_strdup_printf(PACKAGE " - %s - " VERSION, base);
+    free(tmp);
+    gtk_window_set_title ((GtkWindow *) main_window, title);
+    free(title);
+}
+
+void s_set_filename(const char *fname)
+{
+    filename = fname;
+}
+
+void s_set_crossfade_time(float ct)
+{
+  /*  We're faking them out here.  0.0 isn't really allowable but
+      most people would rather put in 0.0 instead of 0.001.  */
+
+  if (ct == 0.0) ct = 0.001;
+  crossfade_time = ct;
+}
+
+float s_get_crossfade_time()
+{
+  return (crossfade_time);
+}
+
+void s_save_global_int(xmlDocPtr doc, char *symbol, int value)
+{
+    xmlNodePtr root = xmlDocGetRootElement(doc);
+    xmlNodePtr node = xmlNewDocRawNode(doc, NULL, "global", NULL);
+    char tmp[256];
+
+    snprintf(tmp, 255, "%d", value);
+    xmlSetProp(node, "name", symbol);
+    xmlSetProp(node, "value", tmp);
+    xmlAddChild(root, node);
+    node = xmlNewText("\n");
+    xmlAddChild(root, node);
+}
+
+void s_save_global_float(xmlDocPtr doc, char *symbol, float value)
+{
+    xmlNodePtr root = xmlDocGetRootElement(doc);
+    xmlNodePtr node = xmlNewDocRawNode(doc, NULL, "global", NULL);
+    char tmp[256];
+
+    snprintf(tmp, 255, "%f", value);
+    xmlSetProp(node, "name", symbol);
+    xmlSetProp(node, "value", tmp);
+    xmlAddChild(root, node);
+    node = xmlNewText("\n");
+    xmlAddChild(root, node);
+}
+
+void s_save_global_gang(xmlDocPtr doc, char *p, int band, gboolean value)
+{
+    xmlNodePtr root = xmlDocGetRootElement(doc);
+    xmlNodePtr node = xmlNewDocRawNode(doc, NULL, "global", NULL);
+    char tmp[256];
+
+    snprintf(tmp, 255, "gang_%s", p);
+    xmlSetProp(node, "name", tmp);
+    snprintf(tmp, 255, "%d", band);
+    xmlSetProp(node, "index", tmp);
+    snprintf(tmp, 255, "%d", value);
+    xmlSetProp(node, "value", tmp);
+    xmlAddChild(root, node);
+    node = xmlNewText("\n");
+    xmlAddChild(root, node);
+}
+
 /* vi:set ts=8 sts=4 sw=4: */
